import redis
from hwhpykit.cache.reids.base.RedisGEO import RedisGEO
from hwhpykit.cache.reids.base.RedisSetManager import RedisSetManager
from hwhpykit.cache.reids.base.RedisHashManager import RedisHashManager
from hwhpykit.cache.reids.base.RedisListManager import RedisListManager
from hwhpykit.cache.reids.base.RedisZSetManager import RedisZSetManager
from hwhpykit.cache.reids.base.RedisStringManager import RedisStringManager
from hwhpykit.cache.reids.action.RedisServer import RedisServer
from hwhpykit.cache.reids.action.RedisSubscriber import RedisSubscriber
from hwhpykit.cache.reids.action.RedisTransaction import RedisTransaction


class RedisManager(object):
    __redis = None
    __pool = None
    __tag = ""

    string = RedisStringManager
    hash = RedisHashManager
    set = RedisSetManager
    zset = RedisZSetManager
    list = RedisListManager
    geo = RedisGEO
    transaction = RedisTransaction
    subscriber = RedisSubscriber
    server = RedisServer

    @classmethod
    def config(cls, host='127.0.0.1', port=6379, db=0, password=''):
        cls.__pool = redis.ConnectionPool(host=host, port=port, db=db, password=password)
        cls.__redis = redis.Redis(connection_pool=cls.__pool)

        cls.geo = RedisGEO.config(cls.__redis)
        cls.set = RedisSetManager.config(cls.__redis)
        cls.hash = RedisHashManager.config(cls.__redis)
        cls.zset = RedisZSetManager.config(cls.__redis)
        cls.list = RedisListManager.config(cls.__redis)
        cls.string = RedisStringManager.config(cls.__redis)

        cls.transaction = RedisTransaction.config(cls.__redis)
        cls.subscriber = RedisSubscriber.config(cls.__redis)
        cls.server = RedisServer.config(cls.__redis)
        return cls







class RedisTransaction(object):
    __redis = None
    __pool = None

    @classmethod
    def config(cls, redis, key=None):
        cls.__redis = redis
        cls.__key = key
        return cls

    @classmethod
    def discard(cls): ...

    @classmethod
    def exec(cls): ...

    @classmethod
    def multi(cls): ...

    @classmethod
    def unwatch(cls): ...

    @classmethod
    def watch(cls, keys): ...






class RedisServer(object):
    __redis = None

    @classmethod
    def config(cls, redis):
        cls.__redis = redis
        return cls

    # @classmethod
    # def aof_save(cls): ...
    #
    # @classmethod
    # def bg_save(cls): ...

    @classmethod
    def client_kill(cls, address):
        return cls.__redis.client_kill(address)

    @classmethod
    def clients(cls, type='normal'):
        return cls.__redis.client_list(type)

    @classmethod
    def unwatch(cls): ...

    @classmethod
    def watch(cls, keys): ...





class RedisListManager(object):
    __redis = None
    __key = None

    @classmethod
    def config(cls, redis, key=None):
        cls.__redis = redis
        cls.__key = key
        return cls

    @classmethod
    def right_push(cls, key, *values):
        return cls.__redis.rpush(key, *values)

    @classmethod
    def right_exist_push(cls, key, value):
        return cls.__redis.rpushx(key, value)

    @classmethod
    def left_push(cls, key, *values):
        return cls.__redis.lpush(key, *values)

    @classmethod
    def left_exist_push(cls, key, value):
        return cls.__redis.lpushx(key, value)

    @classmethod
    def insert(cls, key, value, before=True, tag=""):
        if before:
            return cls.__redis.linsert(key, "before", tag, value)
        else:
            return cls.__redis.linsert(key, "after", tag, value)

    @classmethod
    def remove_value(cls, key, value, count):
        return cls.__redis.lrem(key,count, value)

    @classmethod
    def right_pop(cls, key):
        return cls.__redis.rpop(key)

    @classmethod
    def left_pop(cls, key):
        return cls.__redis.lpop(key)

    @classmethod
    def right_pop_left_push(cls, origin, target):
        return cls.__redis.rpoplpush(origin, target)

    @classmethod
    def b_right_pop(cls, keys, timeout=0):
        return cls.__redis.brpop(keys, timeout)

    @classmethod
    def b_left_pop(cls, keys, timeout):
        return cls.__redis.blpop(keys, timeout)

    @classmethod
    def b_right_pop_left_push(cls, right_key, left_key, timeout=0):
        return cls.__redis.brpoplpush(right_key, left_key, timeout)

    @classmethod
    def left_set(cls, key, value, index):
        return cls.__redis.lset(key, index, value)

    @classmethod
    def trim_list(cls, key, start, end):
        return cls.__redis.ltrim(key, start, end)

    @classmethod
    def get_index(cls, key, index):
        return cls.__redis.lindex(key, index)

    @classmethod
    def get_range(cls, key, start, end):
        return cls.__redis.lrange(key, start, end)

    @classmethod
    def get_list_count(cls, key):
        return cls.__redis.llen(key)






from tests.ip_data import get_all_user_ip_map
from tests.user_data import get_all_user
from phone import Phone
from hwhpykit.ip.IPTransfer import IPTransfer
import xlwt


def save_data_to_excel(data_list, path):
    # 标题栏背景色
    styleBlueBkg = xlwt.easyxf('pattern: pattern solid, fore_colour pale_blue; font: bold on;');  # 80% like
    # 创建一个工作簿
    book = xlwt.Workbook(encoding='utf-8', style_compression=0)
    # 创建一张表
    sheet = book.add_sheet('用户数据', cell_overwrite_ok=True)
    # 标题栏
    title_list = ('用户Id', '手机号', '手机号归属地', '网络运营商', 'IP', "IP城市")
    # 设置第一列尺寸

    first_col = sheet.col(0)
    first_col.width = 256 * 30
    # 写入标题栏
    for i in range(0, len(title_list)):
        sheet.write(0, i, title_list[i], styleBlueBkg)

    # 写入Chat信息
    for i in range(0, len(data_list)):
        data = data_list[i]
        one_data_list = [
            data.get('id'), data.get('mobile'),
            data.get('mobile_city'), data.get('mobile_type'),
            data.get('ip'), data.get('ip_city')
        ]
        for j in range(0, len(one_data_list)):
            sheet.write(i + 1, j, one_data_list[j])
    book.save(path)


if __name__ == '__main__':
    user_ip_map = get_all_user_ip_map()
    all_users = get_all_user()
    transfer = IPTransfer()
    phoneTransfer = Phone()

    for one in all_users:
        user_id = one['id']
        ip = user_ip_map.get(user_id)
        if ip:
            one['ip'] = ip
            one['ip_city'] = transfer.ip_to_city(ip)
        user_mobile = one.get('mobile')

        if 7 <= len(user_mobile) <= 11:
            info = phoneTransfer.find(user_mobile)
            if info:
                one['mobile_province'] = info.get('province')
                one['mobile_city'] = info.get('city')
                one['mobile_type'] = info.get('phone_type')
                one['mobile_zip_code'] = info.get('zip_code')
                one['city_area_code'] = info.get('area_code')

    print(all_users)
    save_data_to_excel(all_users, "./user.xls")




    # if __name__ == "__main__":
    #     phoneNum = '17613394466'
    #     info = Phone().find(phoneNum)
    #     print(info)
    #     try:
    #         phone = info['phone']
    #         province = info['province']
    #         city = info['city']
    #         zip_code = info['zip_code']
    #         area_code = info['area_code']
    #         phone_type = info['phone_type']
    #     except:
    #         print('none')





country_name_en_map = {
    "AF": "Afghanistan",
    "AFG": "Afghanistan",
    "AX": "Aland Islands",
    "ALA": "Aland Islands",
    "AL": "Albania",
    "ALB": "Albania",
    "DZ": "Algeria",
    "DZA": "Algeria",
    "AS": "American Samoa",
    "ASM": "American Samoa",
    "AD": "Andorra",
    "AND": "Andorra",
    "AO": "Angola",
    "AGO": "Angola",
    "AI": "Anguilla",
    "AIA": "Anguilla",
    "AQ": "Antarctica",
    "ATA": "Antarctica",
    "AG": "Antigua and Barbuda",
    "ATG": "Antigua and Barbuda",
    "AR": "Argentina",
    "ARG": "Argentina",
    "AM": "Armenia",
    "ARM": "Armenia",
    "AW": "Aruba",
    "ABW": "Aruba",
    "AU": "Australia",
    "AUS": "Australia",
    "AT": "Austria",
    "AUT": "Austria",
    "AZ": "Azerbaijan",
    "AZE": "Azerbaijan",
    "BS": "Bahamas",
    "BHS": "Bahamas",
    "BH": "Bahrain",
    "BHR": "Bahrain",
    "BD": "Bangladesh",
    "BGD": "Bangladesh",
    "BB": "Barbados",
    "BRB": "Barbados",
    "BY": "Belarus",
    "BLR": "Belarus",
    "BE": "Belgium",
    "BEL": "Belgium",
    "BZ": "Belize",
    "BLZ": "Belize",
    "BJ": "Benin",
    "BEN": "Benin",
    "BM": "Bermuda",
    "BMU": "Bermuda",
    "BT": "Bhutan",
    "BTN": "Bhutan",
    "BO": "Bolivia (Plurinational State of)",
    "BOL": "Bolivia (Plurinational State of)",
    "BQ": "Bonaire, Sint Eustatius and Saba",
    "BES": "Bonaire, Sint Eustatius and Saba",
    "BA": "Bosnia and Herzegovina",
    "BIH": "Bosnia and Herzegovina",
    "BW": "Botswana",
    "BWA": "Botswana",
    "BV": "Bouvet Island",
    "BVT": "Bouvet Island",
    "BR": "Brazil",
    "BRA": "Brazil",
    "IO": "British Indian Ocean Territory",
    "IOT": "British Indian Ocean Territory",
    "BN": "Brunei Darussalam",
    "BRN": "Brunei Darussalam",
    "BG": "Bulgaria",
    "BGR": "Bulgaria",
    "BF": "Burkina Faso",
    "BFA": "Burkina Faso",
    "BI": "Burundi",
    "BDI": "Burundi",
    "CV": "Cabo Verde",
    "CPV": "Cabo Verde",
    "KH": "Cambodia",
    "KHM": "Cambodia",
    "CM": "Cameroon",
    "CMR": "Cameroon",
    "CA": "Canada",
    "CAN": "Canada",
    "KY": "Cayman Islands",
    "CYM": "Cayman Islands",
    "CF": "Central African Republic",
    "CAF": "Central African Republic",
    "TD": "Chad",
    "TCD": "Chad",
    "CL": "Chile",
    "CHL": "Chile",
    "CN": "China",
    "CHN": "China",
    "CX": "Christmas Island",
    "CXR": "Christmas Island",
    "CC": "Cocos (Keeling) Islands",
    "CCK": "Cocos (Keeling) Islands",
    "CO": "Colombia",
    "COL": "Colombia",
    "KM": "Comoros",
    "COM": "Comoros",
    "CG": "Congo",
    "COG": "Congo",
    "CD": "Congo (Democratic Republic of the)",
    "COD": "Congo (Democratic Republic of the)",
    "CK": "Cook Islands",
    "COK": "Cook Islands",
    "CR": "Costa Rica",
    "CRI": "Costa Rica",
    "CI": "Côte d'Ivoire",
    "CIV": "Côte d'Ivoire",
    "HR": "Croatia",
    "HRV": "Croatia",
    "CU": "Cuba",
    "CUB": "Cuba",
    "CW": "Curaçao",
    "CUW": "Curaçao",
    "CY": "Cyprus",
    "CYP": "Cyprus",
    "CZ": "Czechia",
    "CZE": "Czechia",
    "DK": "Denmark",
    "DNK": "Denmark",
    "DJ": "Djibouti",
    "DJI": "Djibouti",
    "DM": "Dominica",
    "DMA": "Dominica",
    "DO": "Dominican Republic",
    "DOM": "Dominican Republic",
    "EC": "Ecuador",
    "ECU": "Ecuador",
    "EG": "Egypt",
    "EGY": "Egypt",
    "SV": "El Salvador",
    "SLV": "El Salvador",
    "GQ": "Equatorial Guinea",
    "GNQ": "Equatorial Guinea",
    "ER": "Eritrea",
    "ERI": "Eritrea",
    "EE": "Estonia",
    "EST": "Estonia",
    "ET": "Ethiopia",
    "ETH": "Ethiopia",
    "FK": "Falkland Islands (Malvinas)",
    "FLK": "Falkland Islands (Malvinas)",
    "FO": "Faroe Islands",
    "FRO": "Faroe Islands",
    "FJ": "Fiji",
    "FJI": "Fiji",
    "FI": "Finland",
    "FIN": "Finland",
    "FR": "France",
    "FRA": "France",
    "GF": "French Guiana",
    "GUF": "French Guiana",
    "PF": "French Polynesia",
    "PYF": "French Polynesia",
    "TF": "French Southern Territories",
    "ATF": "French Southern Territories",
    "GA": "Gabon",
    "GAB": "Gabon",
    "GM": "Gambia",
    "GMB": "Gambia",
    "GE": "Georgia",
    "GEO": "Georgia",
    "DE": "Germany",
    "DEU": "Germany",
    "GH": "Ghana",
    "GHA": "Ghana",
    "GI": "Gibraltar",
    "GIB": "Gibraltar",
    "GR": "Greece",
    "GRC": "Greece",
    "GL": "Greenland",
    "GRL": "Greenland",
    "GD": "Grenada",
    "GRD": "Grenada",
    "GP": "Guadeloupe",
    "GLP": "Guadeloupe",
    "GU": "Guam",
    "GUM": "Guam",
    "GT": "Guatemala",
    "GTM": "Guatemala",
    "GG": "Guernsey",
    "GGY": "Guernsey",
    "GN": "Guinea",
    "GIN": "Guinea",
    "GW": "Guinea-Bissau",
    "GNB": "Guinea-Bissau",
    "GY": "Guyana",
    "GUY": "Guyana",
    "HT": "Haiti",
    "HTI": "Haiti",
    "HM": "Heard Island and McDonald Islands",
    "HMD": "Heard Island and McDonald Islands",
    "VA": "Holy See",
    "VAT": "Holy See",
    "HN": "Honduras",
    "HND": "Honduras",
    "HK": "Hong Kong",
    "HKG": "Hong Kong",
    "HU": "Hungary",
    "HUN": "Hungary",
    "IS": "Iceland",
    "ISL": "Iceland",
    "IN": "India",
    "IND": "India",
    "ID": "Indonesia",
    "IDN": "Indonesia",
    "IR": "Iran (Islamic Republic of)",
    "IRN": "Iran (Islamic Republic of)",
    "IQ": "Iraq",
    "IRQ": "Iraq",
    "IE": "Ireland",
    "IRL": "Ireland",
    "IM": "Isle of Man",
    "IMN": "Isle of Man",
    "IL": "Israel",
    "ISR": "Israel",
    "IT": "Italy",
    "ITA": "Italy",
    "JM": "Jamaica",
    "JAM": "Jamaica",
    "JP": "Japan",
    "JPN": "Japan",
    "JE": "Jersey",
    "JEY": "Jersey",
    "JO": "Jordan",
    "JOR": "Jordan",
    "KZ": "Kazakhstan",
    "KAZ": "Kazakhstan",
    "KE": "Kenya",
    "KEN": "Kenya",
    "KI": "Kiribati",
    "KIR": "Kiribati",
    "KP": "Korea (Democratic People's Republic of)",
    "PRK": "Korea (Democratic People's Republic of)",
    "KR": "Korea (Republic of)",
    "KOR": "Korea (Republic of)",
    "KW": "Kuwait",
    "KWT": "Kuwait",
    "KG": "Kyrgyzstan",
    "KGZ": "Kyrgyzstan",
    "LA": "Lao People's Democratic Republic",
    "LAO": "Lao People's Democratic Republic",
    "LV": "Latvia",
    "LVA": "Latvia",
    "LB": "Lebanon",
    "LBN": "Lebanon",
    "LS": "Lesotho",
    "LSO": "Lesotho",
    "LR": "Liberia",
    "LBR": "Liberia",
    "LY": "Libya",
    "LBY": "Libya",
    "LI": "Liechtenstein",
    "LIE": "Liechtenstein",
    "LT": "Lithuania",
    "LTU": "Lithuania",
    "LU": "Luxembourg",
    "LUX": "Luxembourg",
    "MO": "Macao",
    "MAC": "Macao",
    "MK": "Macedonia (the former Yugoslav Republic of)",
    "MKD": "Macedonia (the former Yugoslav Republic of)",
    "MG": "Madagascar",
    "MDG": "Madagascar",
    "MW": "Malawi",
    "MWI": "Malawi",
    "MY": "Malaysia",
    "MYS": "Malaysia",
    "MV": "Maldives",
    "MDV": "Maldives",
    "ML": "Mali",
    "MLI": "Mali",
    "MT": "Malta",
    "MLT": "Malta",
    "MH": "Marshall Islands",
    "MHL": "Marshall Islands",
    "MQ": "Martinique",
    "MTQ": "Martinique",
    "MR": "Mauritania",
    "MRT": "Mauritania",
    "MU": "Mauritius",
    "MUS": "Mauritius",
    "YT": "Mayotte",
    "MYT": "Mayotte",
    "MX": "Mexico",
    "MEX": "Mexico",
    "FM": "Micronesia (Federated States of)",
    "FSM": "Micronesia (Federated States of)",
    "MD": "Moldova (Republic of)",
    "MDA": "Moldova (Republic of)",
    "MC": "Monaco",
    "MCO": "Monaco",
    "MN": "Mongolia",
    "MNG": "Mongolia",
    "ME": "Montenegro",
    "MNE": "Montenegro",
    "MS": "Montserrat",
    "MSR": "Montserrat",
    "MA": "Morocco",
    "MAR": "Morocco",
    "MZ": "Mozambique",
    "MOZ": "Mozambique",
    "MM": "Myanmar",
    "MMR": "Myanmar",
    "NA": "Namibia",
    "NAM": "Namibia",
    "NR": "Nauru",
    "NRU": "Nauru",
    "NP": "Nepal",
    "NPL": "Nepal",
    "NL": "Netherlands",
    "NLD": "Netherlands",
    "NC": "New Caledonia",
    "NCL": "New Caledonia",
    "NZ": "New Zealand",
    "NZL": "New Zealand",
    "NI": "Nicaragua",
    "NIC": "Nicaragua",
    "NE": "Niger",
    "NER": "Niger",
    "NG": "Nigeria",
    "NGA": "Nigeria",
    "NU": "Niue",
    "NIU": "Niue",
    "NF": "Norfolk Island",
    "NFK": "Norfolk Island",
    "MP": "Northern Mariana Islands",
    "MNP": "Northern Mariana Islands",
    "NO": "Norway",
    "NOR": "Norway",
    "OM": "Oman",
    "OMN": "Oman",
    "PK": "Pakistan",
    "PAK": "Pakistan",
    "PW": "Palau",
    "PLW": "Palau",
    "PS": "Palestine, State of",
    "PSE": "Palestine, State of",
    "PA": "Panama",
    "PAN": "Panama",
    "PG": "Papua New Guinea",
    "PNG": "Papua New Guinea",
    "PY": "Paraguay",
    "PRY": "Paraguay",
    "PE": "Peru",
    "PER": "Peru",
    "PH": "Philippines",
    "PHL": "Philippines",
    "PN": "Pitcairn",
    "PCN": "Pitcairn",
    "PL": "Poland",
    "POL": "Poland",
    "PT": "Portugal",
    "PRT": "Portugal",
    "PR": "Puerto Rico",
    "PRI": "Puerto Rico",
    "QA": "Qatar",
    "QAT": "Qatar",
    "RE": "Réunion",
    "REU": "Réunion",
    "RO": "Romania",
    "ROU": "Romania",
    "RU": "Russian Federation",
    "RUS": "Russian Federation",
    "RW": "Rwanda",
    "RWA": "Rwanda",
    "BL": "Saint Barthélemy",
    "BLM": "Saint Barthélemy",
    "SH": "Saint Helena, Ascension and Tristan da Cunha",
    "SHN": "Saint Helena, Ascension and Tristan da Cunha",
    "KN": "Saint Kitts and Nevis",
    "KNA": "Saint Kitts and Nevis",
    "LC": "Saint Lucia",
    "LCA": "Saint Lucia",
    "MF": "Saint Martin (French part)",
    "MAF": "Saint Martin (French part)",
    "PM": "Saint Pierre and Miquelon",
    "SPM": "Saint Pierre and Miquelon",
    "VC": "Saint Vincent and the Grenadines",
    "VCT": "Saint Vincent and the Grenadines",
    "WS": "Samoa",
    "WSM": "Samoa",
    "SM": "San Marino",
    "SMR": "San Marino",
    "ST": "Sao Tome and Principe",
    "STP": "Sao Tome and Principe",
    "SA": "Saudi Arabia",
    "SAU": "Saudi Arabia",
    "SN": "Senegal",
    "SEN": "Senegal",
    "RS": "Serbia",
    "SRB": "Serbia",
    "SC": "Seychelles",
    "SYC": "Seychelles",
    "SL": "Sierra Leone",
    "SLE": "Sierra Leone",
    "SG": "Singapore",
    "SGP": "Singapore",
    "SX": "Sint Maarten (Dutch part)",
    "SXM": "Sint Maarten (Dutch part)",
    "SK": "Slovakia",
    "SVK": "Slovakia",
    "SI": "Slovenia",
    "SVN": "Slovenia",
    "SB": "Solomon Islands",
    "SLB": "Solomon Islands",
    "SO": "Somalia",
    "SOM": "Somalia",
    "ZA": "South Africa",
    "ZAF": "South Africa",
    "GS": "South Georgia and the South Sandwich Islands",
    "SGS": "South Georgia and the South Sandwich Islands",
    "SS": "South Sudan",
    "SSD": "South Sudan",
    "ES": "Spain",
    "ESP": "Spain",
    "LK": "Sri Lanka",
    "LKA": "Sri Lanka",
    "SD": "Sudan",
    "SDN": "Sudan",
    "SR": "Suriname",
    "SUR": "Suriname",
    "SJ": "Svalbard and Jan Mayen",
    "SJM": "Svalbard and Jan Mayen",
    "SZ": "Swaziland",
    "SWZ": "Swaziland",
    "SE": "Sweden",
    "SWE": "Sweden",
    "CH": "Switzerland",
    "CHE": "Switzerland",
    "SY": "Syrian Arab Republic",
    "SYR": "Syrian Arab Republic",
    "TW": "Taiwan, Province of China[note 1]",
    "TWN": "Taiwan, Province of China[note 1]",
    "TJ": "Tajikistan",
    "TJK": "Tajikistan",
    "TZ": "Tanzania, United Republic of",
    "TZA": "Tanzania, United Republic of",
    "TH": "Thailand",
    "THA": "Thailand",
    "TL": "Timor-Leste",
    "TLS": "Timor-Leste",
    "TG": "Togo",
    "TGO": "Togo",
    "TK": "Tokelau",
    "TKL": "Tokelau",
    "TO": "Tonga",
    "TON": "Tonga",
    "TT": "Trinidad and Tobago",
    "TTO": "Trinidad and Tobago",
    "TN": "Tunisia",
    "TUN": "Tunisia",
    "TR": "Turkey",
    "TUR": "Turkey",
    "TM": "Turkmenistan",
    "TKM": "Turkmenistan",
    "TC": "Turks and Caicos Islands",
    "TCA": "Turks and Caicos Islands",
    "TV": "Tuvalu",
    "TUV": "Tuvalu",
    "UG": "Uganda",
    "UGA": "Uganda",
    "UA": "Ukraine",
    "UKR": "Ukraine",
    "AE": "United Arab Emirates",
    "ARE": "United Arab Emirates",
    "GB": "United Kingdom of Great Britain and Northern Ireland",
    "GBR": "United Kingdom of Great Britain and Northern Ireland",
    "US": "United States of America",
    "USA": "United States of America",
    "UM": "United States Minor Outlying Islands",
    "UMI": "United States Minor Outlying Islands",
    "UY": "Uruguay",
    "URY": "Uruguay",
    "UZ": "Uzbekistan",
    "UZB": "Uzbekistan",
    "VU": "Vanuatu",
    "VUT": "Vanuatu",
    "VE": "Venezuela (Bolivarian Republic of)",
    "VEN": "Venezuela (Bolivarian Republic of)",
    "VN": "Viet Nam",
    "VNM": "Viet Nam",
    "VG": "Virgin Islands (British)",
    "VGB": "Virgin Islands (British)",
    "VI": "Virgin Islands (U.S.)",
    "VIR": "Virgin Islands (U.S.)",
    "WF": "Wallis and Futuna",
    "WLF": "Wallis and Futuna",
    "EH": "Western Sahara",
    "ESH": "Western Sahara",
    "YE": "Yemen",
    "YEM": "Yemen",
    "ZM": "Zambia",
    "ZMB": "Zambia",
    "ZW": "Zimbabwe",
    "ZWE": "Zimbabwe"
}

country_name_map = {
    "AF": "阿富汗",
    "AFG": "阿富汗",
    "AX": "奥兰",
    "ALA": "奥兰",
    "AL": "阿尔巴尼亚",
    "ALB": "阿尔巴尼亚",
    "DZ": "阿尔及利亚",
    "DZA": "阿尔及利亚",
    "AS": "美属萨摩亚",
    "ASM": "美属萨摩亚",
    "AD": "安道尔",
    "AND": "安道尔",
    "AO": "安哥拉",
    "AGO": "安哥拉",
    "AI": "安圭拉",
    "AIA": "安圭拉",
    "AQ": "南极洲",
    "ATA": "南极洲",
    "AG": "安提瓜和巴布达",
    "ATG": "安提瓜和巴布达",
    "AR": "阿根廷",
    "ARG": "阿根廷",
    "AM": "亚美尼亚",
    "ARM": "亚美尼亚",
    "AW": "阿鲁巴",
    "ABW": "阿鲁巴",
    "AU": "澳大利亚",
    "AUS": "澳大利亚",
    "AT": "奥地利",
    "AUT": "奥地利",
    "AZ": "阿塞拜疆",
    "AZE": "阿塞拜疆",
    "BS": "巴哈马",
    "BHS": "巴哈马",
    "BH": "巴林",
    "BHR": "巴林",
    "BD": "孟加拉国",
    "BGD": "孟加拉国",
    "BB": "巴巴多斯",
    "BRB": "巴巴多斯",
    "BY": "白俄罗斯",
    "BLR": "白俄罗斯",
    "BE": "比利时",
    "BEL": "比利时",
    "BZ": "伯利兹",
    "BLZ": "伯利兹",
    "BJ": "贝宁",
    "BEN": "贝宁",
    "BM": "百慕大",
    "BMU": "百慕大",
    "BT": "不丹",
    "BTN": "不丹",
    "BO": "玻利维亚",
    "BOL": "玻利维亚",
    "BQ": "荷兰加勒比区",
    "BES": "荷兰加勒比区",
    "BA": "波斯尼亚和黑塞哥维那",
    "BIH": "波斯尼亚和黑塞哥维那",
    "BW": "博茨瓦纳",
    "BWA": "博茨瓦纳",
    "BV": "布韦岛",
    "BVT": "布韦岛",
    "BR": "巴西",
    "BRA": "巴西",
    "IO": "英属印度洋领地",
    "IOT": "英属印度洋领地",
    "BN": "文莱",
    "BRN": "文莱",
    "BG": "保加利亚",
    "BGR": "保加利亚",
    "BF": "布基纳法索",
    "BFA": "布基纳法索",
    "BI": "布隆迪",
    "BDI": "布隆迪",
    "CV": "佛得角",
    "CPV": "佛得角",
    "KH": "柬埔寨",
    "KHM": "柬埔寨",
    "CM": "喀麦隆",
    "CMR": "喀麦隆",
    "CA": "加拿大",
    "CAN": "加拿大",
    "KY": "开曼群岛",
    "CYM": "开曼群岛",
    "CF": "中非",
    "CAF": "中非",
    "TD": "乍得",
    "TCD": "乍得",
    "CL": "智利",
    "CHL": "智利",
    "CN": "中国",
    "CHN": "中国",
    "CX": "圣诞岛",
    "CXR": "圣诞岛",
    "CC": "科科斯（基林）群岛",
    "CCK": "科科斯（基林）群岛",
    "CO": "哥伦比亚",
    "COL": "哥伦比亚",
    "KM": "科摩罗",
    "COM": "科摩罗",
    "CG": "刚果（布）",
    "COG": "刚果（布）",
    "CD": "刚果（金）",
    "COD": "刚果（金）",
    "CK": "库克群岛",
    "COK": "库克群岛",
    "CR": "哥斯达黎加",
    "CRI": "哥斯达黎加",
    "CI": "科特迪瓦",
    "CIV": "科特迪瓦",
    "HR": "克罗地亚",
    "HRV": "克罗地亚",
    "CU": "古巴",
    "CUB": "古巴",
    "CW": "库拉索",
    "CUW": "库拉索",
    "CY": "塞浦路斯",
    "CYP": "塞浦路斯",
    "CZ": "捷克",
    "CZE": "捷克",
    "DK": "丹麦",
    "DNK": "丹麦",
    "DJ": "吉布提",
    "DJI": "吉布提",
    "DM": "多米尼克",
    "DMA": "多米尼克",
    "DO": "多米尼加",
    "DOM": "多米尼加",
    "EC": "厄瓜多尔",
    "ECU": "厄瓜多尔",
    "EG": "埃及",
    "EGY": "埃及",
    "SV": "萨尔瓦多",
    "SLV": "萨尔瓦多",
    "GQ": "赤道几内亚",
    "GNQ": "赤道几内亚",
    "ER": "厄立特里亚",
    "ERI": "厄立特里亚",
    "EE": "爱沙尼亚",
    "EST": "爱沙尼亚",
    "ET": "埃塞俄比亚",
    "ETH": "埃塞俄比亚",
    "FK": "福克兰群岛",
    "FLK": "福克兰群岛",
    "FO": "法罗群岛",
    "FRO": "法罗群岛",
    "FJ": "斐济",
    "FJI": "斐济",
    "FI": "芬兰",
    "FIN": "芬兰",
    "FR": "法国",
    "FRA": "法国",
    "GF": "法属圭亚那",
    "GUF": "法属圭亚那",
    "PF": "法属波利尼西亚",
    "PYF": "法属波利尼西亚",
    "TF": "法属南方和南极洲领地",
    "ATF": "法属南方和南极洲领地",
    "GA": "加蓬",
    "GAB": "加蓬",
    "GM": "冈比亚",
    "GMB": "冈比亚",
    "GE": "格鲁吉亚",
    "GEO": "格鲁吉亚",
    "DE": "德国",
    "DEU": "德国",
    "GH": "加纳",
    "GHA": "加纳",
    "GI": "直布罗陀",
    "GIB": "直布罗陀",
    "GR": "希腊",
    "GRC": "希腊",
    "GL": "格陵兰",
    "GRL": "格陵兰",
    "GD": "格林纳达",
    "GRD": "格林纳达",
    "GP": "瓜德罗普",
    "GLP": "瓜德罗普",
    "GU": "关岛",
    "GUM": "关岛",
    "GT": "危地马拉",
    "GTM": "危地马拉",
    "GG": "根西",
    "GGY": "根西",
    "GN": "几内亚",
    "GIN": "几内亚",
    "GW": "几内亚比绍",
    "GNB": "几内亚比绍",
    "GY": "圭亚那",
    "GUY": "圭亚那",
    "HT": "海地",
    "HTI": "海地",
    "HM": "赫德岛和麦克唐纳群岛",
    "HMD": "赫德岛和麦克唐纳群岛",
    "VA": "梵蒂冈",
    "VAT": "梵蒂冈",
    "HN": "洪都拉斯",
    "HND": "洪都拉斯",
    "HK": "香港",
    "HKG": "香港",
    "HU": "匈牙利",
    "HUN": "匈牙利",
    "IS": "冰岛",
    "ISL": "冰岛",
    "IN": "印度",
    "IND": "印度",
    "ID": "印尼",
    "IDN": "印尼",
    "IR": "伊朗",
    "IRN": "伊朗",
    "IQ": "伊拉克",
    "IRQ": "伊拉克",
    "IE": "爱尔兰",
    "IRL": "爱尔兰",
    "IM": "马恩岛",
    "IMN": "马恩岛",
    "IL": "以色列",
    "ISR": "以色列",
    "IT": "意大利",
    "ITA": "意大利",
    "JM": "牙买加",
    "JAM": "牙买加",
    "JP": "日本",
    "JPN": "日本",
    "JE": "泽西",
    "JEY": "泽西",
    "JO": "约旦",
    "JOR": "约旦",
    "KZ": "哈萨克斯坦",
    "KAZ": "哈萨克斯坦",
    "KE": "肯尼亚",
    "KEN": "肯尼亚",
    "KI": "基里巴斯",
    "KIR": "基里巴斯",
    "KP": "朝鲜",
    "PRK": "朝鲜",
    "KR": "韩国",
    "KOR": "韩国",
    "KW": "科威特",
    "KWT": "科威特",
    "KG": "吉尔吉斯斯坦",
    "KGZ": "吉尔吉斯斯坦",
    "LA": "老挝",
    "LAO": "老挝",
    "LV": "拉脱维亚",
    "LVA": "拉脱维亚",
    "LB": "黎巴嫩",
    "LBN": "黎巴嫩",
    "LS": "莱索托",
    "LSO": "莱索托",
    "LR": "利比里亚",
    "LBR": "利比里亚",
    "LY": "利比亚",
    "LBY": "利比亚",
    "LI": "列支敦士登",
    "LIE": "列支敦士登",
    "LT": "立陶宛",
    "LTU": "立陶宛",
    "LU": "卢森堡",
    "LUX": "卢森堡",
    "MO": "澳门",
    "MAC": "澳门",
    "MK": "马其顿",
    "MKD": "马其顿",
    "MG": "马达加斯加",
    "MDG": "马达加斯加",
    "MW": "马拉维",
    "MWI": "马拉维",
    "MY": "马来西亚",
    "MYS": "马来西亚",
    "MV": "马尔代夫",
    "MDV": "马尔代夫",
    "ML": "马里",
    "MLI": "马里",
    "MT": "马耳他",
    "MLT": "马耳他",
    "MH": "马绍尔群岛",
    "MHL": "马绍尔群岛",
    "MQ": "马提尼克",
    "MTQ": "马提尼克",
    "MR": "毛里塔尼亚",
    "MRT": "毛里塔尼亚",
    "MU": "毛里求斯",
    "MUS": "毛里求斯",
    "YT": "马约特",
    "MYT": "马约特",
    "MX": "墨西哥",
    "MEX": "墨西哥",
    "FM": "密克罗尼西亚联邦",
    "FSM": "密克罗尼西亚联邦",
    "MD": "摩尔多瓦",
    "MDA": "摩尔多瓦",
    "MC": "摩纳哥",
    "MCO": "摩纳哥",
    "MN": "蒙古国",
    "MNG": "蒙古国",
    "ME": "黑山",
    "MNE": "黑山",
    "MS": "蒙特塞拉特",
    "MSR": "蒙特塞拉特",
    "MA": "摩洛哥",
    "MAR": "摩洛哥",
    "MZ": "莫桑比克",
    "MOZ": "莫桑比克",
    "MM": "缅甸",
    "MMR": "缅甸",
    "NA": "纳米比亚",
    "NAM": "纳米比亚",
    "NR": "瑙鲁",
    "NRU": "瑙鲁",
    "NP": "尼泊尔",
    "NPL": "尼泊尔",
    "NL": "荷兰",
    "NLD": "荷兰",
    "NC": "新喀里多尼亚",
    "NCL": "新喀里多尼亚",
    "NZ": "新西兰",
    "NZL": "新西兰",
    "NI": "尼加拉瓜",
    "NIC": "尼加拉瓜",
    "NE": "尼日尔",
    "NER": "尼日尔",
    "NG": "尼日利亚",
    "NGA": "尼日利亚",
    "NU": "纽埃",
    "NIU": "纽埃",
    "NF": "诺福克岛",
    "NFK": "诺福克岛",
    "MP": "北马里亚纳群岛",
    "MNP": "北马里亚纳群岛",
    "NO": "挪威",
    "NOR": "挪威",
    "OM": "阿曼",
    "OMN": "阿曼",
    "PK": "巴基斯坦",
    "PAK": "巴基斯坦",
    "PW": "帕劳",
    "PLW": "帕劳",
    "PS": "巴勒斯坦",
    "PSE": "巴勒斯坦",
    "PA": "巴拿马",
    "PAN": "巴拿马",
    "PG": "巴布亚新几内亚",
    "PNG": "巴布亚新几内亚",
    "PY": "巴拉圭",
    "PRY": "巴拉圭",
    "PE": "秘鲁",
    "PER": "秘鲁",
    "PH": "菲律宾",
    "PHL": "菲律宾",
    "PN": "皮特凯恩群岛",
    "PCN": "皮特凯恩群岛",
    "PL": "波兰",
    "POL": "波兰",
    "PT": "葡萄牙",
    "PRT": "葡萄牙",
    "PR": "波多黎各",
    "PRI": "波多黎各",
    "QA": "卡塔尔",
    "QAT": "卡塔尔",
    "RE": "留尼汪",
    "REU": "留尼汪",
    "RO": "罗马尼亚",
    "ROU": "罗马尼亚",
    "RU": "俄罗斯",
    "RUS": "俄罗斯",
    "RW": "卢旺达",
    "RWA": "卢旺达",
    "BL": "圣巴泰勒米",
    "BLM": "圣巴泰勒米",
    "SH": "圣赫勒拿、阿森松和特里斯坦-达库尼亚",
    "SHN": "圣赫勒拿、阿森松和特里斯坦-达库尼亚",
    "KN": "圣基茨和尼维斯",
    "KNA": "圣基茨和尼维斯",
    "LC": "圣卢西亚",
    "LCA": "圣卢西亚",
    "MF": "法属圣马丁",
    "MAF": "法属圣马丁",
    "PM": "圣皮埃尔和密克隆",
    "SPM": "圣皮埃尔和密克隆",
    "VC": "圣文森特和格林纳丁斯",
    "VCT": "圣文森特和格林纳丁斯",
    "WS": "萨摩亚",
    "WSM": "萨摩亚",
    "SM": "圣马力诺",
    "SMR": "圣马力诺",
    "ST": "圣多美和普林西比",
    "STP": "圣多美和普林西比",
    "SA": "沙特阿拉伯",
    "SAU": "沙特阿拉伯",
    "SN": "塞内加尔",
    "SEN": "塞内加尔",
    "RS": "塞尔维亚",
    "SRB": "塞尔维亚",
    "SC": "塞舌尔",
    "SYC": "塞舌尔",
    "SL": "塞拉利昂",
    "SLE": "塞拉利昂",
    "SG": "新加坡",
    "SGP": "新加坡",
    "SX": "圣马丁",
    "SXM": "圣马丁",
    "SK": "斯洛伐克",
    "SVK": "斯洛伐克",
    "SI": "斯洛文尼亚",
    "SVN": "斯洛文尼亚",
    "SB": "所罗门群岛",
    "SLB": "所罗门群岛",
    "SO": "索马里",
    "SOM": "索马里",
    "ZA": "南非",
    "ZAF": "南非",
    "GS": "南乔治亚和南桑威奇群岛",
    "SGS": "南乔治亚和南桑威奇群岛",
    "SS": "南苏丹",
    "SSD": "南苏丹",
    "ES": "西班牙",
    "ESP": "西班牙",
    "LK": "斯里兰卡",
    "LKA": "斯里兰卡",
    "SD": "苏丹",
    "SDN": "苏丹",
    "SR": "苏里南",
    "SUR": "苏里南",
    "SJ": "斯瓦尔巴和扬马延",
    "SJM": "斯瓦尔巴和扬马延",
    "SZ": "斯威士兰",
    "SWZ": "斯威士兰",
    "SE": "瑞典",
    "SWE": "瑞典",
    "CH": "瑞士",
    "CHE": "瑞士",
    "SY": "叙利亚",
    "SYR": "叙利亚",
    "TW": "中国台湾省",
    "TWN": "中国台湾省",
    "TJ": "塔吉克斯坦",
    "TJK": "塔吉克斯坦",
    "TZ": "坦桑尼亚",
    "TZA": "坦桑尼亚",
    "TH": "泰国",
    "THA": "泰国",
    "TL": "东帝汶",
    "TLS": "东帝汶",
    "TG": "多哥",
    "TGO": "多哥",
    "TK": "托克劳",
    "TKL": "托克劳",
    "TO": "汤加",
    "TON": "汤加",
    "TT": "特立尼达和多巴哥",
    "TTO": "特立尼达和多巴哥",
    "TN": "突尼斯",
    "TUN": "突尼斯",
    "TR": "土耳其",
    "TUR": "土耳其",
    "TM": "土库曼斯坦",
    "TKM": "土库曼斯坦",
    "TC": "特克斯和凯科斯群岛",
    "TCA": "特克斯和凯科斯群岛",
    "TV": "图瓦卢",
    "TUV": "图瓦卢",
    "UG": "乌干达",
    "UGA": "乌干达",
    "UA": "乌克兰",
    "UKR": "乌克兰",
    "AE": "阿联酋",
    "ARE": "阿联酋",
    "GB": "英国",
    "GBR": "英国",
    "US": "美国",
    "USA": "美国",
    "UM": "美国本土外小岛屿",
    "UMI": "美国本土外小岛屿",
    "UY": "乌拉圭",
    "URY": "乌拉圭",
    "UZ": "乌兹别克斯坦",
    "UZB": "乌兹别克斯坦",
    "VU": "瓦努阿图",
    "VUT": "瓦努阿图",
    "VE": "委内瑞拉",
    "VEN": "委内瑞拉",
    "VN": "越南",
    "VNM": "越南",
    "VG": "英属维尔京群岛",
    "VGB": "英属维尔京群岛",
    "VI": "美属维尔京群岛",
    "VIR": "美属维尔京群岛",
    "WF": "瓦利斯和富图纳",
    "WLF": "瓦利斯和富图纳",
    "EH": "西撒哈拉",
    "ESH": "西撒哈拉",
    "YE": "也门",
    "YEM": "也门",
    "ZM": "赞比亚",
    "ZMB": "赞比亚",
    "ZW": "津巴布韦",
    "ZWE": "津巴布韦",
}




class RedisZSetManager(object):
    __redis = None
    __pool = None

    @classmethod
    def config(cls, redis, key=None):
        cls.__redis = redis
        cls.__key = key
        return cls

    @classmethod
    def add(cls, key, members_score):
        return cls.__redis.zadd(key, members_score)

    @classmethod
    def count(cls, key):
        """总数量"""
        return cls.__redis.zcard(key)

    @classmethod
    def increment_member(cls, key, increment, member):
        return cls.__redis.zincrby(key, increment, member)

    @classmethod
    def range_count(cls, key, min_element="-", max_element="+"):
        """ 两个成员之间元素的数量 """
        return cls.__redis.zlexcount(key, min=min_element, max=max_element)

    @classmethod
    def range_values(cls, key, start, stop, desc=False, withscores=False):
        """ 在集合中元素的排名 """
        return cls.__redis.zrange(key, start, stop, desc=desc, withscores=withscores)

    @classmethod
    def range_values_by_lex(cls, key, min_element="-", max_element="+", desc=False):
        """ 两个成员之间的元素 """
        if desc:
            return cls.__redis.zrevrangebylex(key, min_element, max_element)
        else:
            return cls.__redis.zrangebylex(key, min_element, max_element)

    @classmethod
    def range_values_by_score(cls, key, min_score, max_score, withscores=False, desc=False):
        """ 两个成员之间的元素 """
        if desc:
            return cls.__redis.zrevrangebyscore(key, min_score, max_score, withscores=withscores)
        else:
            return cls.__redis.zrangebyscore(key, min_score, max_score, withscores=withscores)

    @classmethod
    def rank(cls, key, member, desc=False):
        """ 在集合中元素的排名 """
        if desc:
            return cls.__redis.zrevrank(key, member)
        else:
            return cls.__redis.zrank(key, member)

    @classmethod
    def remove(cls, key, *members):
        return cls.__redis.zrem(key, *members)

    @classmethod
    def remove_range_by_lex(cls, key, min_element, max_element):
        return cls.__redis.zremrangebylex(key, min=min_element, max=max_element)

    @classmethod
    def remove_range_by_score(cls, key, min_score, max_score):
        return cls.__redis.zremrangebyscore(key, min_score, max_score)

    @classmethod
    def remove_range_by_rank(cls, key, min_, max_):
        return cls.__redis.zremrangebyrank(key, min_, max_)

    @classmethod
    def get_score(cls, key, member):
        return cls.__redis.zscore(key, member)

    @classmethod
    def generate_difference(cls, gen_key, keys):
        return cls.__redis.zinterstore(gen_key, keys=keys)

    @classmethod
    def generate_union(cls, key, *keys):
        return cls.__redis.zunionstore(key, *keys)

    @classmethod
    def scan(cls, key, cursor=0, match=None, count=None):
        return cls.__redis.zscan(key, cursor, match, count)





class RedisHashManager(object):
    __redis = None
    __key = None

    @classmethod
    def config(cls, redis, key=None):
        cls.__redis = redis
        cls.__key = key
        return cls

    @classmethod
    def set_map(cls, key, dic):
        cls.__redis.hset(key, mapping=dic)

    @classmethod
    def set_value(cls, key, field, value):
        cls.__redis.hset(key, key=field, value=value)

    @classmethod
    def set_not_exits_value(cls, key, field, value):
        cls.__redis.hsetnx(key, key=field, value=value)

    @classmethod
    def delete_field(cls, key, field):
        cls.__redis.hdel(key, field)

    @classmethod
    def increase_field_int(cls, key, field, increment=1):
        cls.__redis.hincrby(key, field, increment)

    @classmethod
    def increase_field_float(cls, key, field, increment=1.0):
        cls.__redis.hincrbyfloat(key, field, increment)

    @classmethod
    def exist_field(cls, key, field):
        return cls.__redis.hexists(key, field)

    @classmethod
    def get_value(cls, key, field):
        return cls.__redis.hget(key, field)

    @classmethod
    def get_values(cls, keys):
        return cls.__redis.hmget(keys)

    @classmethod
    def get_all(cls, key):
        return cls.__redis.hgetall(key)

    @classmethod
    def get_all_values(cls, key):
        return cls.__redis.hvals(key)

    @classmethod
    def get_all_keys(cls, key):
        return cls.__redis.hkeys(key)

    @classmethod
    def get_all_key_count(cls, key):
        return cls.__redis.hlen(key)

    @classmethod
    def get_value_bytes_len(cls, key, field):
        return cls.__redis.hstrlen(key, key=field)

    @classmethod
    def scan(cls, key, cursor, pattern, count):
        return cls.__redis.hscan(key, cursor, pattern, count)








class RedisGEO(object):
    __redis = None
    __pool = None

    @classmethod
    def config(cls, redis, key=None):
        cls.__redis = redis
        cls.__key = key
        return cls

    @classmethod
    def add(cls, key, *values):
        return cls.__redis.geoadd(key, *values)

    @classmethod
    def get_location(cls, key, member):
        return cls.__redis.geopos(key, member)

    @classmethod
    def distance(cls, key, member1, member2, unit='km'):
        return cls.__redis.geodist(key, member1, member2, unit=unit)

    @classmethod
    def radius_locations(cls, key, longitude, latitude, radius,
                         unit='km', withdist=False, withcoord=False,
                         count=None,sort=None):
        return cls.__redis.georadius(name=key, longitude=longitude, latitude=latitude,
                                     radius=radius, unit=unit, withdist=withdist,
                                     withcoord=withcoord, count=count, sort=sort)

    @classmethod
    def radius_locations_by_member(cls, key, member, radius,
                         unit=None, withdist=False, withcoord=False,
                         count=None,sort=None):
        return cls.__redis.georadiusbymember(name=key, member=member,radius=radius,
                                             unit=unit, withdist=withdist,
                                             withcoord=withcoord, count=count, sort=sort)

    @classmethod
    def hash(cls, key, *members):
        return cls.__redis.geohash(key, *members)






import datetime
import pymysql


def singleton(cls):
    _instance = {}

    def _singleton(*args, **kargs):
        if cls not in _instance:
            _instance[cls] = cls(*args, **kargs)
        return _instance[cls]
    return _singleton


@singleton
class MysqlManager(object):
    """docstring for DBManager"""

    def __init__(self, db_host, db_user, db_pass, db_name, db_port=3306):
        self.db_host = db_host
        self.db_user = db_user
        self.db_pass = db_pass
        self.db_name = db_name
        self.db_port = db_port
        self._add_db()

    def _add_db(self):
        self.db = pymysql.connect(
            host=self.db_host,
            port=self.db_port,
            user=self.db_user,
            passwd=self.db_pass,
            db=self.db_name,
            charset='utf8mb4')
        self.cursor = self.db.cursor()

    def execute(self, sql):
        try:
            self.cursor.execute(sql)
            self.db.commit()
            return self.cursor
        except Exception as e:
            print("---rollback--{}".format(e))
            self.db.rollback()
            self._add_db()

    def select(self, sql):
        try:
            self.cursor.execute(sql)
            self.db.commit()
            return self.cursor.fetchall()
        except Exception as e:
            print("Error")
            return None




class RedisSetManager(object):
    __redis = None

    @classmethod
    def config(cls, redis):
        cls.__redis = redis
        return cls

    @classmethod
    def add(cls, key, *values):
        return cls.__redis.sadd(key, *values)

    @classmethod
    def remove(cls, key, *values):
        return cls.__redis.srem(key, *values)

    @classmethod
    def count(cls, key):
        return cls.__redis.scard(key)

    @classmethod
    def difference(cls, keys, *args):
        return cls.__redis.sdiff(keys, *args)

    @classmethod
    def generate_difference(cls, gen_key, *keys):
        return cls.__redis.sdiffstore(gen_key, keys)

    @classmethod
    def intersection(cls, *keys):
        return cls.__redis.sinter(keys)

    @classmethod
    def generate_intersection(cls, gen_key, *keys):
        return cls.__redis.sinterstore(gen_key, keys)

    @classmethod
    def union(cls, *keys):
        return cls.__redis.sunion(*keys)

    @classmethod
    def generate_union(cls, gen_key, *keys):
        return cls.__redis.sunionstore(gen_key, keys)

    @classmethod
    def is_member(cls, value, key):
        return cls.__redis.sismember(key, value)

    @classmethod
    def move(cls, value, from_, to_):
        return cls.__redis.smove(from_, to_, value)

    @classmethod
    def pop(cls, key, count=None):
        return cls.__redis.spop(key, count)

    @classmethod
    def get_all_value(cls, key):
        return cls.__redis.smembers(key)

    @classmethod
    def get_random_value(cls, key, count=None):
        return cls.__redis.srandmember(key, count)

    @classmethod
    def scan(cls, key, cursor=0, pattern=None, count=None):
        return cls.__redis.sscan(key, cursor, pattern, count)




import paho.mqtt.client as mqtt
import asyncio
import functools
from typing import Callable, Any, Dict, List


class MQTTAsyncClient:
    _instance = None

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
            cls._instance._initialized = False
        return cls._instance

    def __init__(self, broker: str = None, port: int = None, client_id: str = None, username: str = None, password: str = None):
        if not self._initialized:
            self.broker = broker
            self.port = port
            self.client_id = client_id
            self.client = mqtt.Client(client_id=client_id)
            if username and password:
                self.client.username_pw_set(username, password)
            self.client.on_connect = self.on_connect
            self.client.on_disconnect = self.on_disconnect
            self.client.reconnect_delay_set(min_delay=1, max_delay=120)
            self._callbacks: Dict[str, List[Callable[[str], Any]]] = {}
            self._initialized = True
            self.loop = asyncio.get_event_loop()

    def on_connect(self, client: mqtt.Client, userdata: Any, flags: dict, rc: int):
        if rc == 0:
            print("Connected to MQTT Broker!")
            for topic in self._callbacks.keys():
                client.subscribe(topic)
        else:
            print(f"Failed to connect, return code {rc}")

    def on_disconnect(self, client: mqtt.Client, userdata: Any, rc: int):
        print("Disconnected from MQTT Broker!")
        if rc != 0:
            print(f"Unexpected disconnection. Reconnecting...")

    def start(self):
        self.client.connect(self.broker, self.port, 60)
        self.client.loop_start()

    def subscribe(self, topic: str, callback: Callable[[str], Any]):
        if topic not in self._callbacks:
            self._callbacks[topic] = []
        self._callbacks[topic].append(callback)
        self.client.message_callback_add(topic, lambda client, userdata, msg: self._on_message(topic, msg))
        print(f"Subscribed to topic: {topic}")

    def _on_message(self, topic: str, msg: mqtt.MQTTMessage):
        print(f"Received message from topic: {topic}")
        for callback in self._callbacks.get(topic, []):
            asyncio.run_coroutine_threadsafe(callback(msg.payload.decode('utf-8')), self.loop)

    def publish(self, topic: str, message: str, qos: int = 0, retain: bool = False):
        result = self.client.publish(topic, message, qos=qos, retain=retain)
        if result.rc == mqtt.MQTT_ERR_SUCCESS:
            print(f"Message published to {topic}")
        else:
            print(f"Failed to publish message to {topic}, error code: {result.rc}")


def mqtt_subscribe(topic: str):
    def decorator(func: Callable[[str], Any]):
        @functools.wraps(func)
        async def wrapper(*args, **kwargs):
            return await func(*args, **kwargs)

        client = MQTTAsyncClient()
        client.subscribe(topic, wrapper)
        return wrapper

    return decorator


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='hwhpykit',
    version='1.1.6',
    description='Packaging tools for own use',
    author='louishwh',
    author_email='louishwh@gmail.com',
    url='',
    #packages=['hwhpykit.connection'],
    packages=setuptools.find_packages(),
    long_description=long_description,
    long_description_content_type="text/markdown",
    python_requires='>=3.10',
    install_requires = [
        'redis>=3.5.3',
        'pymysql>=1.1.0',
        'paho-mqtt>=2.0.0'
    ],
    classifiers=[  # PyPI 分类
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)






class RedisSubscriber(object):
    __redis = None
    __pool = None

    @classmethod
    def config(cls, redis, key=None):
        cls.__redis = redis
        cls.__key = key
        return cls

    @classmethod
    def subscribe_by_pattern(cls, *patterns):
        cls.__redis.psubscribe(*patterns)

    @classmethod
    def unsubscribe_by_pattern(cls, *patterns):
        cls.__redis.punsubscribe(*patterns)

    @classmethod
    def subscribe_by_channel(cls, *channels):
        cls.__redis.subscribe(*channels)

    @classmethod
    def unsubscribe_by_channel(cls, *channels):
        cls.__redis.unsubscribe(*channels)

    @classmethod
    def scribe_status(cls, *channels):
        cls.__redis.pubsub(*channels)

    @classmethod
    def publish_to_channel(cls, channel, message):
        cls.__redis.publish(channel, message)






class RedisStringManager(object):
    __redis = None
    __key = None

    @classmethod
    def config(cls, redis, key=None):
        cls.__redis = redis
        cls.__key = key
        return cls

    # Set ----------------------------------------------

    @classmethod
    def set_not_exist_key(cls, key, value):
        return cls.__redis.setnx(key, value)

    @classmethod
    def set_not_exist_keys(cls, kv_list):
        return cls.__redis.msetnx(kv_list)

    @classmethod
    def set(cls, key, value, seconds=None):
        if seconds is None:
            return cls.__redis.set(key, value)
        else:
            if seconds > 0:
                return cls.__redis.setex(key, seconds, value)
            else:
                return cls.__redis.psetex(key, seconds * 1000, value)

    @classmethod
    def set_keys(cls, kv_list):
        return cls.__redis.mset(kv_list)

    @classmethod
    def set_bit(cls, key, index, bit):
        return cls.__redis.setbit(key, index, bit)

    @classmethod
    def set_range(cls, key, start, value):
        return cls.__redis.setrange(key, start, value)

    # Get ----------------------------------------------

    @classmethod
    def get(cls, key, start=None, end=None):
        if start is None and end is None:
            return cls.__redis.get(key)
        else:
            return cls.__redis.getrange(key, start, end)

    @classmethod
    def get_len(cls, key):
        return cls.__redis.strlen(key)

    @classmethod
    def get_range(cls, key, start, end):
        return cls.__redis.getrange(key, start, end)

    @classmethod
    def get_set(cls, key, new_value):
        return cls.__redis.getset(key, new_value)

    @classmethod
    def get_bit(cls, key, index):
        return cls.__redis.getbit(key, index)

    @classmethod
    def get_values(cls, keys):
        return cls.__redis.mget(keys)

    # Other Options ----------------------------------------------

    @classmethod
    def append(cls, key, value):
        return cls.__redis.append(key, value)

    @classmethod
    def increase(cls, key, count=None):
        if count is None:
            return cls.__redis.incr(key)
        else:
            return cls.__redis.incrby(key, count)

    @classmethod
    def decrease(cls, key, count=None):
        if count is None:
            return cls.__redis.decr(key)
        else:
            return cls.__redis.decrby(key, count)





